//-----------------------------------------------------------------------------
// MK3_2560_TIMER
// Version 1.1
// 16.06.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board (oder MK2 Board)
// mit ATMega-2560 @ 16MHz (oder ATMega8)
// Timer Funktionen
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_TIMER.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : @MK3 (ATMega2560)
//            Timer-0
// Belegt : @MK2 (ATMega8)
//            Timer-2
//
// Hinweis :
//
// > die DEFINE-Schalter "TIMER_HW_MODE" und "TIMER_QUARZ_TYP"
//   muessen zur Hardware passen !!
//-----------------------------------------------------------------------------


#include <avr/interrupt.h>

//-----------------------------------------------------------------------------
// Hier koennen Funktionen die nicht benutzt werden
// abgeschaltet werden...einfach den DEFINE von "1" auf "0" aendern
//-----------------------------------------------------------------------------
#define USE_MK3_WAIT_US  1
#define USE_MK3_WAIT_MS  1
#define USE_MK3_WAIT_S   1
#define USE_MK3_TIMER_US 1
#define USE_MK3_TIMER_MS 1
#define USE_MK3_TIMER_S  1



//-----------------------------------------------------------------------------
// Hardware auswahl
//-----------------------------------------------------------------------------
#define TIMER_HW_MODE    0       // 0=MK3-Board, 1=MK1/MK2-Board/mySmartControl
#define TIMER_QUARZ_TYP  0       // 0=16MHz, 1=8MHz, 2=3.6864MHz


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_TIMER" starten
// 02. Funktion "MK3_WAIT_US" wartet einige mikrosekunden
// 03. Funktion "MK3_WAIT_MS" wartet einige millisekunden
// 04. Funktion "MK3_WAIT_S" wartet einige sekunden
// 05a. Funktion "MK3_TIMER_START_US" startet Timer fuer mikrosekunden
// 05b. Funktion "MK3_TIMER_START_MS" startet Timer fuer millisekunden
// 05c. Funktion "MK3_TIMER_START_S" startet Timer fuer sekunden
// 06. Funktion "MK3_TIMER_STOP" stoppt Timer
// 07. Funktion "MK3_TIMER_CHECK" test ob Timer abgelaufen
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------

typedef enum {
  TIMER_IDLE =0, // Mode = IDLE
  TIMER_STOP,    // Mode = STOP
  TIMER_US,      // Mode = mikro sekunden
  TIMER_MS,      // Mode = milli sekunden
  TIMER_S        // Mode = sekunden
}TIMERTYP;


//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_TIMER(void);                      // zum initialisieren
void MK3_WAIT_US(unsigned int pause_us);        // wartet einige mikrosekunden
void MK3_WAIT_MS(unsigned int pause_ms);        // wartet einige millisekunden
void MK3_WAIT_S(unsigned int pause_s);          // wartet einige sekunden
void MK3_TIMER_START_US(unsigned int timer_us); // startet Timer fuer mikrosekunden
void MK3_TIMER_START_MS(unsigned int timer_ms); // startet Timer fuer millisekunden
void MK3_TIMER_START_S(unsigned int timer_s);   // startet Timer fuer sekunden
void MK3_TIMER_STOP(void);                      // stoppt Timer
bool MK3_TIMER_CHECK(void);                     // test ob Timer abgelaufen


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
// keine 

//-----------------------------------------------------------------------------
// interne Globale Variabeln
//-----------------------------------------------------------------------------
typedef struct {
  TIMERTYP akt_mode;
  unsigned char step;  
  unsigned int akt_us;
  unsigned int akt_ms;  
  unsigned int akt_s;  
  unsigned int soll_us;
  unsigned int soll_ms;  
  unsigned int soll_s; 
}Timer_Intern_t;
volatile Timer_Intern_t TIMER;


//-----------------------------------------------------------------------------
void MK3_INIT_TIMER(void) {	// zum initialisieren

  // alle Variablen initialisieren
  TIMER.akt_mode=TIMER_IDLE;
  TIMER.step=10;
  TIMER.akt_us=0;
  TIMER.akt_ms=0; 
  TIMER.akt_s=0;  
  TIMER.soll_us=0;
  TIMER.soll_ms=0; 
  TIMER.soll_s=0;

  #if TIMER_HW_MODE==0
    // Hardware = myAVR MK3-Board

    // benutzter Timer : Timer-0
    // im Mode2 -> ClearTimerOnCompare-A Match

    TCCR0A=0x02;         // Mode2 (CTC)
    TCCR0B=0x00;

    TCNT0=0x00;    
    OCR0B=0x00;          // Comp-B wird nicht benutzt

    TIMSK0=0x02;         // Interrupt bei Comp-A 

    #if TIMER_QUARZ_TYP==0
      // 16MHz
      OCR0A=159;
      TCCR0B|=0x01; // Vorteiler=1, OCR=159 -> 10.00us
      TIMER.step=10;
    #elif TIMER_QUARZ_TYP==1
      // 8MHz
      OCR0A=199;	  
      TCCR0B|=0x01; // Vorteiler=1, OCR=199 -> 25.00us
      TIMER.step=25;
    #elif TIMER_QUARZ_TYP==2
      // 3.6864MHz
      OCR0A=183;
      TCCR0B|=0x01; // Vorteiler=1, OCR=183 -> 49.91us
      TIMER.step=50;
    #endif    
  #else
    // Hardware = myAVR MK1/MK2-Board + mySmartControl

    // benutzter Timer : Timer-2
    // im Mode2 -> ClearTimerOnCompare Match

    TCCR2=0x08;         // Mode2 (CTC)
    TCNT2=0x00;

    TIMSK=0x80;         // Interrupt bei Comp

    #if TIMER_QUARZ_TYP==0
      // 16MHz
      OCR2=159;
      TCCR2|=0x01; // Vorteiler=1, OCR=159 -> 10.00us 
      TIMER.step=10;
    #elif TIMER_QUARZ_TYP==1
      // 8MHz
      OCR2=199;	  
      TCCR2|=0x01; // Vorteiler=1, OCR=199 -> 25.00us
      TIMER.step=25;
    #elif TIMER_QUARZ_TYP==2
      // 3.6864MHz
      OCR2=183;
      TCCR2|=0x01; // Vorteiler=1, OCR=183 -> 49.91us
      TIMER.step=50;
    #endif    
  #endif

  SREG|=0x80; // Alle Interrupts einschalten
}


//-----------------------------------------------------------------------------
#if USE_MK3_WAIT_US==1
void MK3_WAIT_US(unsigned int pause_us) { // wartet einige mikrosekunden
  // pausenzeit [0 mikrosek bis 65535 mikrosek]

  if(TIMER.akt_mode!=TIMER_IDLE) {  
    // timer stoppen falls notwendig
    TIMER.akt_mode=TIMER_STOP;
    // warten bis timer gestoppt
    while(TIMER.akt_mode!=TIMER_IDLE) {
      // warten
    }
  }
  
  // timer starten
  TIMER.akt_mode=TIMER_US;
  #if TIMER_HW_MODE==0
    TCCR0B|=0x01;
  #else
    TCCR2|=0x01;
  #endif
  // warten bis zeit abgelaufen
  while(TIMER.akt_us<pause_us) {
    // warten
  }
  // timer stoppen
  TIMER.akt_mode=TIMER_STOP;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_WAIT_MS==1
void MK3_WAIT_MS(unsigned int pause_ms) { // wartet einige millisekunden
  // pausenzeit [0 millisek bis 65535 millisek]

  if(TIMER.akt_mode!=TIMER_IDLE) {  
    // timer stoppen falls notwendig
    TIMER.akt_mode=TIMER_STOP;
    // warten bis timer gestoppt
    while(TIMER.akt_mode!=TIMER_IDLE) {
      // warten
    }
  }

  // timer starten
  TIMER.akt_mode=TIMER_MS;
  #if TIMER_HW_MODE==0
    TCCR0B|=0x01;
  #else
    TCCR2|=0x01;
  #endif
  // warten bis zeit abgelaufen
  while(TIMER.akt_ms<pause_ms) {
    // warten
  }
  // timer stoppen
  TIMER.akt_mode=TIMER_STOP;	
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_WAIT_S==1
void MK3_WAIT_S(unsigned int pause_s) { // wartet einige sekunden
  // pausenzeit [0 sek bis 65535 sek]

  if(TIMER.akt_mode!=TIMER_IDLE) {  
    // timer stoppen falls notwendig
    TIMER.akt_mode=TIMER_STOP;
    // warten bis timer gestoppt
    while(TIMER.akt_mode!=TIMER_IDLE) {
      // warten
    }
  }

  // timer starten
  TIMER.akt_mode=TIMER_S;
  #if TIMER_HW_MODE==0
    TCCR0B|=0x01;
  #else
    TCCR2|=0x01;
  #endif
  // warten bis zeit abgelaufen
  while(TIMER.akt_s<pause_s) {
    // warten
  }
  // timer stoppen
  TIMER.akt_mode=TIMER_STOP;
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_TIMER_US==1
void MK3_TIMER_START_US(unsigned int timer_us) { // startet Timer fuer mikrosekunden
  // timerzeit [0 mikrosek bis 65535 mikrosek]

  if(TIMER.akt_mode!=TIMER_IDLE) {  
    // timer stoppen falls notwendig
    TIMER.akt_mode=TIMER_STOP;
    // warten bis timer gestoppt
    while(TIMER.akt_mode!=TIMER_IDLE) {
      // warten
    }
  }        
  // Timer vorbereiten
  TIMER.soll_us=timer_us;
  // Timer starten
  TIMER.akt_mode=TIMER_US;
  #if TIMER_HW_MODE==0
    TCCR0B|=0x01;
  #else
    TCCR2|=0x01;
  #endif
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_TIMER_MS==1
void MK3_TIMER_START_MS(unsigned int timer_ms) { // startet Timer fuer millisekunden
  // timerzeit [0 millisek bis 65535 millisek]
  
  if(TIMER.akt_mode!=TIMER_IDLE) {  
    // timer stoppen falls notwendig
    TIMER.akt_mode=TIMER_STOP;
    // warten bis timer gestoppt
    while(TIMER.akt_mode!=TIMER_IDLE) {
      // warten
    }
  }  
  // Timer vorbereiten
  TIMER.soll_ms=timer_ms;
  // Timer starten
  TIMER.akt_mode=TIMER_MS;
  #if TIMER_HW_MODE==0
    TCCR0B|=0x01;
  #else
    TCCR2|=0x01;
  #endif  
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_TIMER_S==1
void MK3_TIMER_START_S(unsigned int timer_s) { // startet Timer fuer sekunden
  // timerzeit [0 sek bis 65535 sek]
  
  if(TIMER.akt_mode!=TIMER_IDLE) {  
    // timer stoppen falls notwendig
    TIMER.akt_mode=TIMER_STOP;
    // warten bis timer gestoppt
    while(TIMER.akt_mode!=TIMER_IDLE) {
      // warten
    }
  }        
  // Timer vorbereiten
  TIMER.soll_s=timer_s;
  // Timer starten
  TIMER.akt_mode=TIMER_S;
  #if TIMER_HW_MODE==0
    TCCR0B|=0x01;
  #else
    TCCR2|=0x01;
  #endif  
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_TIMER_US==1 || USE_MK3_TIMER_MS==1 || USE_MK3_TIMER_S==1
void MK3_TIMER_STOP(void) { // stoppt Timer
  if(TIMER.akt_mode!=TIMER_IDLE) {  
    // timer stoppen
    TIMER.akt_mode=TIMER_STOP;
  }    
}
#endif

//-----------------------------------------------------------------------------
#if USE_MK3_TIMER_US==1 || USE_MK3_TIMER_MS==1 || USE_MK3_TIMER_S==1
bool MK3_TIMER_CHECK(void) { // test ob Timer abgelaufen
  // ret_wert : false = timer noch nicht abgelaufen
  //            true  = timer ist abgelaufen

  bool ret_wert=false;

  if((TIMER.akt_mode==TIMER_IDLE) || (TIMER.akt_mode==TIMER_STOP)) {
    // Timer ist gestoppt
    ret_wert=true;    
  }  
  else if(TIMER.akt_mode==TIMER_US) {
    if(TIMER.akt_us>=TIMER.soll_us) {
      // wenn zeit abgelaufen
      ret_wert=true;
      // Timer stoppen
      TIMER.akt_mode=TIMER_STOP;     
    }    
  }
  else if(TIMER.akt_mode==TIMER_MS) {
    if(TIMER.akt_ms>=TIMER.soll_ms) {
      // wenn zeit abgelaufen
      ret_wert=true;
      // Timer stoppen
      TIMER.akt_mode=TIMER_STOP;      
    }
  }
  else if(TIMER.akt_mode==TIMER_S) {
    if(TIMER.akt_s>=TIMER.soll_s) {
      // wenn zeit abgelaufen
      ret_wert=true;
      // Timer stoppen
      TIMER.akt_mode=TIMER_STOP;      
    } 
  }

  return(ret_wert);
}
#endif


#if TIMER_HW_MODE==0
// Hardware = myAVR MK3-Board
//-----------------------------------------------------------------------------
// ISR von Timer-0 Compare-A
//-----------------------------------------------------------------------------
ISR (TIMER0_COMPA_vect) {                          // ISR : Timer2 Comp-A
  // Beim Comp-A interrupt wird
  // der Timer automatisch wieder auf 0 gesetzt
  // 
  // der Interrupt im Interval aufgerufen
  // step = 10, 25, 50 us  

  if(TIMER.akt_mode==TIMER_IDLE) {
    // timer Abschalten
    TCCR0B&=~0x07;	
  }
  else if(TIMER.akt_mode==TIMER_STOP) {
    // timer soll gestoppt werden
    TIMER.akt_us=0;
    TIMER.akt_ms=0;
    TIMER.akt_s=0;
    TIMER.akt_mode=TIMER_IDLE;
  }
  else if(TIMER.akt_mode==TIMER_US) {
    // im mikrosekunden Mode
    TIMER.akt_us+=TIMER.step;
  }
  else if(TIMER.akt_mode==TIMER_MS) {
    // im millisekunden Mode
    TIMER.akt_us+=TIMER.step;
    if(TIMER.akt_us>=1000) {
      // eine ms um
      TIMER.akt_us=0;
      TIMER.akt_ms+=1;
    }
  }
  else if(TIMER.akt_mode==TIMER_S) {
    // im sekunden Mode
    TIMER.akt_us+=TIMER.step;
    if(TIMER.akt_us>=1000) {
      // eine ms um
      TIMER.akt_us=0;
      TIMER.akt_ms+=1;
      if(TIMER.akt_ms>=1000) {
        // eine sek um
        TIMER.akt_ms=0;
        TIMER.akt_s+=1;
      }
    }
  }
}
#else
// Hardware = myAVR MK1/MK2-Board + mySmartControl
//-----------------------------------------------------------------------------
// ISR von Timer-2 Compare
//-----------------------------------------------------------------------------
ISR (TIMER2_COMP_vect) {                          // ISR : Timer2 Comp
  // Beim Comp interrupt wird
  // der Timer automatisch wieder auf 0 gesetzt
  // 
  // der Interrupt im Interval aufgerufen
  // step = 10, 25, 50 us 
 
  if(TIMER.akt_mode==TIMER_IDLE) {
    // timer Abschalten
    TCCR2&=~0x07;
  }
  else if(TIMER.akt_mode==TIMER_STOP) {
    // timer soll gestoppt werden
    TIMER.akt_us=0;
    TIMER.akt_ms=0;
    TIMER.akt_s=0;
    TIMER.akt_mode=TIMER_IDLE;
  }
  else if(TIMER.akt_mode==TIMER_US) {
    // im mikrosekunden Mode
    TIMER.akt_us+=TIMER.step;
  }
  else if(TIMER.akt_mode==TIMER_MS) {
    // im millisekunden Mode
    TIMER.akt_us+=TIMER.step;
    if(TIMER.akt_us>=1000) {
      // eine ms um
      TIMER.akt_us=0;
      TIMER.akt_ms+=1;
    }
  }
  else if(TIMER.akt_mode==TIMER_S) {
    // im sekunden Mode
    TIMER.akt_us+=TIMER.step;
    if(TIMER.akt_us>=1000) {
      // eine ms um
      TIMER.akt_us=0;
      TIMER.akt_ms+=1;
      if(TIMER.akt_ms>=1000) {
        // eine sek um
        TIMER.akt_ms=0;
        TIMER.akt_s+=1;
      }
    }
  }
}
#endif
